`define FREQ_62M5		0		//pr=3, od=7
`define FREQ_75M		1		//pr=0, od=7
`define FREQ_100M		2		//pr=0, od=5
`define FREQ_125M		3		//pr=3, od=3
`define FREQ_150M		4		//pr=0, od=3
`define FREQ_156M25	5		//pr=2, od=3
`define FREQ_187M5	6		//pr=1, od=1
`define FREQ_200M		7		//pr=0, od=2
`define FREQ_250M		8		//pr=3, od=1
`define FREQ_312M5	9		//pr=2, od=1
`define FREQ_625M		10		//pr=2, od=0


module CDCM6100x_Config(
	clk_50m,
	recal_n,
	desired_freq,
	
	// CDCM pin
	CLK_CE,
	CLK_OD,
	CLK_OS,
	CLK_PR,
	CLK_RST_n	
);

input								clk_50m;
input								recal_n;
input				 [3:0]		desired_freq;

output		          		CLK_CE;
output		     [2:0]		CLK_OD;
output		     [1:0]		CLK_OS;
output		     [1:0]		CLK_PR;
output		          		CLK_RST_n;

/////////////////////////////////////////


/////////////////////////////////////////
// Device Reset (internal)
reg DEVICE_RESET_n = 0; // internal
reg [19:0] reset_cnt = 0;

always @(posedge clk_50m)
begin
	if (~DEVICE_RESET_n)
	begin
		if (reset_cnt <= 250_000) // 5 ms
			reset_cnt <= reset_cnt + 1;
		else
			DEVICE_RESET_n <= 1'b1;
		
	end
end

/////////////////////////////////////////
// Device Reset (external) trigger by falling edge

reg RECALIBRATION_n = 1'b1; // externl
reg [3:0] recal_cnt = 0;

always @(posedge clk_50m or negedge recal_n)
begin
	if (~recal_n)
	begin
		recal_cnt <= 0;
		RECALIBRATION_n <= 1'b0;
	end
	else
	begin
		if (recal_cnt <= 5) // 100 ns
			recal_cnt <= recal_cnt + 1;
		else
			RECALIBRATION_n <= 1'b1;
		
	end
end

/////////////////////////////////////////
reg	[1:0] pr;
reg	[2:0] od;

always @(*)
begin
	case(desired_freq)
		`FREQ_62M5:
			begin
				pr <= 3;
				od <= 7;
			end
		`FREQ_75M:
			begin
				pr <= 0;
				od <= 7;
			end
		`FREQ_100M:
			begin
				pr <= 0;
				od <= 5;
			end
		`FREQ_125M:
			begin
				pr <= 3;
				od <= 3;
			end
		`FREQ_150M:
			begin
				pr <= 0;
				od <= 3;
			end
		`FREQ_156M25:
			begin
				pr <= 2;
				od <= 3;
			end
		`FREQ_187M5:
			begin
				pr <= 1;
				od <= 1;
			end
		`FREQ_200M:
			begin
				pr <= 0;
				od <= 2;
			end
		`FREQ_250M:
			begin
				pr <= 3;
				od <= 1;
			end
		`FREQ_312M5:
			begin
				pr <= 2;
				od <= 1;
			end
		`FREQ_625M:
			begin
				pr <= 2;
				od <= 0;
			end
		default: // 150mhz
			begin
				pr <= 0;
				od <= 3;
			end
   endcase
end



/////////////////////////////////////////

assign CLK_CE = 1'b1;
assign CLK_OD = od;
assign CLK_PR = pr;
assign CLK_OS = 2'b01; // LVDS 
assign CLK_RST_n = DEVICE_RESET_n & RECALIBRATION_n;

endmodule
