// --------------------------------------------------------------------
// Copyright (c) 2012 by Terasic Technologies Inc.
// --------------------------------------------------------------------
//
// Permission:
//
//   Terasic grants permission to use and modify this code for use
//   in synthesis for all Terasic Development Boards and Altera Development
//   Kits made by Terasic.  Other use of this code, including the selling
//   ,duplication, or modification of any portion is strictly prohibited.
//
// Disclaimer:
//
//   This VHDL/Verilog or C/C++ source code is intended as a design reference
//   which illustrates how these types of functions can be implemented.
//   It is the user's responsibility to verify their design for
//   consistency and functionality through the use of formal
//   verification methods.  Terasic provides no warranty regarding the use
//   or functionality of this code.
//
// --------------------------------------------------------------------
//
//                     Terasic Technologies Inc
//                     E. Rd Sec. 1. JhuBei City,
//                     HsinChu County, Taiwan
//                     302
//
//                     web: http://www.terasic.com/
//                     email: support@terasic.com
//
// --------------------------------------------------------------------

#include "terasic_includes.h"

#define LED_STAY_DUR 	(150*1000)

typedef struct{
	alt_u8 mask;
	bool left_shift;
	alt_u8 led_num;
	alt_u32 avmm_base;
}LED_STATUS;

void LED_Play(LED_STATUS *pStatus){

	IOWR(pStatus->avmm_base, 0x00, ~pStatus->mask);

	if (pStatus->left_shift){
		pStatus->mask <<= 1;
		if (pStatus->mask == (0x01 << pStatus->led_num)){
			pStatus->mask = 0x01 << (pStatus->led_num-1);
			pStatus->left_shift = FALSE;
		}
	}else{
		pStatus->mask >>= 1;
		if (pStatus->mask == 0x00){
			pStatus->mask = 0x01;
			pStatus->left_shift = TRUE;
		}
	}
}

void DisplayInfo(void){
	alt_u8 *pData = (alt_u8 *)EXT_FLASH_BASE + 0x10000;
	int i;
	printf("[Flash]\r\n");
	printf("SN=");
	for(i=0;i<15;i++)
		printf("%c", *(pData+i));
	printf("\r\n");
	printf("TERASIC_END\r\n");
}



int main()
{

#if 0
	DisplayInfo();
	return 0;
#endif


	LED_STATUS LED_USER =    {0x01, TRUE, 3, LED_BASE};
	LED_STATUS LED_BRACKET = {0x01, TRUE, 4, LED_BRACKET_BASE};
	LED_STATUS LED_RJ45 =    {0x01, TRUE, 2, LED_RJ45_BASE};

	printf("Terasic - Parallel Flash Loader!\n");
	while(1){
		// led run
		LED_Play(&LED_USER);
		LED_Play(&LED_BRACKET);
		LED_Play(&LED_RJ45);


		// delay
		usleep(LED_STAY_DUR);

	}

	return 0;
}
